==> BestCrypt-2.0.7.tar.gz: BestCrypt-2.0.7/kernel/kmod/bc_crypto.c <==
/*
 * Copyright (c) 2010 Jetico, Inc., Finland
 * All rights reserved.
 *
 * Interface to crypto library, cipher plugin reference counting, key storage.
 */

#include "BCKernel.h" // for mode definitions

#include "bc_crypto.h"
#include "bc_defs.h"
#include "bc_mod.h" 

#include "crypto/cipher.h"
#include "crypto/mode.h"

#include <linux/spinlock.h>
#include <linux/slab.h>

#include "bc_mem.h"

/**
 * Key table slot.
 * Designed for quick access when user locks the slot to copy key data somewhere, update pointers, etc.
 */
struct bc_key_slot {
    char*       key;
    size_t      keysize;
	spinlock_t	lock;	// lock to protect key addition and access ops.
};

#define BC_LOCK_INIT(slot) spin_lock_init(&(slot)->lock)
#define BC_LOCK_SLOT(slot) spin_lock(&(slot)->lock)
#define BC_UNLOCK_SLOT(slot) spin_unlock(&(slot)->lock)

// static key table data
static struct bc_key_slot g_key_table[kBCKernel_MaxKeys];


static inline int mode2tag(int mode_id) {
	switch (mode_id) {
	case kBCMode_CBC: return kCBC;
	case kBCMode_LRW: return kLRW;
	case kBCMode_XTS: return kXTS;
	default: return -1;
	};
}


static void* crypto_alloc(size_t nbytes) {
    void* mem = KMALLOC(nbytes);
    if(mem) {
        memset(mem, 0, nbytes);
    }

    return mem;
}


static void crypto_free(void* mem, size_t nbytes) {
    memset(mem, 0, nbytes);
    KFREE(mem);
}


static inline int is_free_slot(struct bc_key_slot* slot) {
	return (slot->key == NULL);
}


static inline void free_key_slot(struct bc_key_slot* slot) {
    crypto_free(slot->key, slot->keysize);
	slot->key = NULL;
	slot->keysize = 0;
}


/////////////////////////////////////////////////////////////////////////////////
//
//  Interface
//
/////////////////////////////////////////////////////////////////////////////////

static char g_zero_pool[512] = {0};

void bc_crypto_init_key_table(void)
{
	unsigned i;
	struct bc_key_slot* slot;

	memset(g_key_table, 0, sizeof(g_key_table));
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = g_key_table + i;
		BC_LOCK_INIT(slot);
	}	
}


void bc_crypto_release_key_table(void) 
{
	struct bc_key_slot* slot;
	unsigned i;
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = g_key_table + i;
		if(!is_free_slot(slot)) {
			BC_DEBUG_LOG("bestcrypt: warning: key slot %d not empty on free\n", i);
			free_key_slot(slot);
		}
	}
		
	memset(g_key_table, 0, sizeof(g_key_table));
}


int bc_crypto_store_key(char* key, size_t keysize, int* out_handle)
{
	struct bc_key_slot* slot = g_key_table;
	unsigned i;

	// we first lock and then check if the spot is free to make sure that two concurrent requests wont race for the same free slot.
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = &g_key_table[i];

		BC_LOCK_SLOT(slot);

		if(is_free_slot(slot)) {
			break;
		} else {
			BC_UNLOCK_SLOT(slot);
		}
	}

	if(i == kBCKernel_MaxKeys) {
		return -EBUSY;
	}

	slot->key = key;
	slot->keysize = keysize;

	BC_UNLOCK_SLOT(slot);

	*out_handle = i;
	
	return 0;
}

int bc_crypto_free_key(int handle)
{
	struct bc_key_slot* slot;

    if (handle >= kBCKernel_MaxKeys || handle < 0) {
        	return -EINVAL;
	}

	slot = &g_key_table[handle];

	BC_LOCK_SLOT(slot);

	if(is_free_slot(slot)) {
		BC_UNLOCK_SLOT(slot);
		return -EINVAL;
	}
	
	free_key_slot(slot);
	
    BC_UNLOCK_SLOT(slot);

	return 0;
}


int bc_crypto_get_alg(int alg_id, int mode_id, struct bc_alg* out_alg)
{
	int	mode_tag;
    const struct bc_mode*   mode;
    const struct bc_cipher* cipher;

    if (NULL == out_alg) {
        BC_KERN_ERROR("Invalid argument\n");
        return -EINVAL;
    }

	mode_tag = mode2tag(mode_id);
	if(mode_tag == -1) {
        BC_KERN_ERROR("No such mode id 0x%x\n", mode_id);
		return -EINVAL;
	}

	cipher = bc_get_plugin(alg_id);
	if(!cipher) {
        BC_KERN_ERROR("No such cipher id 0x%x\n", alg_id);
		return -EINVAL;
	}

	mode = bc_lookup_mode(mode_tag);
    if(!mode || !mode->is_supported(cipher)) {
        bc_put_plugin(cipher);
        BC_KERN_ERROR("Mode 0x%x is not supported by cipher 0x%x\n", mode_id, alg_id);
        return -EINVAL;
    }

	memset(out_alg, 0, sizeof(out_alg));

    out_alg->key_size = mode->key_size(cipher);
    out_alg->extended_key_size = mode->context_size(cipher);
	out_alg->cipher = cipher;
	out_alg->mode = mode;
    out_alg->key = NULL;

	return 0;
}


int bc_crypto_set_key(struct bc_alg* alg, int handle)
{
    void* ctx;
	struct bc_key_slot* slot;

	if(handle < 0 || handle >= kBCKernel_MaxKeys || NULL == alg) {
		return -EINVAL;
	}

	slot = &g_key_table[handle];
	
	BC_LOCK_SLOT(slot);

	if(!is_free_slot(slot)) {
        
        if(slot->keysize < alg->key_size) {
            BC_UNLOCK_SLOT(slot);
            return -EINVAL;
        }

        ctx = crypto_alloc(alg->extended_key_size);
        if(!ctx) {
            BC_UNLOCK_SLOT(slot);
            return -EINVAL;
        }

        alg->mode->init_context(ctx, alg->cipher, slot->key, slot->keysize);

        if(alg->key) {
            crypto_free(alg->key, alg->extended_key_size);
        }

        alg->key = ctx;

	} else {
	    BC_UNLOCK_SLOT(slot);
		return -EINVAL;
	}

	BC_UNLOCK_SLOT(slot);

	return 0;
}


int bc_crypto_set_key_explicit(struct bc_alg* alg, char* key, size_t keysize)
{
    void* ctx;

    if(!alg || !key) {
        return -EINVAL;
    }

    if(keysize < alg->key_size) {
        return -EINVAL;
    }

    ctx = crypto_alloc(alg->extended_key_size);
    if(!ctx) {
        return -ENOMEM;
    }

    alg->mode->init_context(ctx, alg->cipher, key, keysize);

    if(alg->key) {
        crypto_free(alg->key, alg->extended_key_size);
    }

    alg->key = ctx;
    return 0;
}


void bc_crypto_encrypt(struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	size_t nblocks = nbytes / alg->cipher->block_size;
	alg->mode->encrypt(alg->key, alg->cipher, iv, (pool ? pool : g_zero_pool), data, nblocks);
}


void bc_crypto_decrypt(struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	size_t nblocks = nbytes / alg->cipher->block_size;
	alg->mode->decrypt(alg->key, alg->cipher, iv, (pool ? pool : g_zero_pool), data, nblocks);
}


void bc_crypto_put_alg(struct bc_alg* alg)
{
	bc_put_plugin(alg->cipher);
    if(alg->key) {
        crypto_free(alg->key, alg->extended_key_size);
    }

    memset(alg, 0, sizeof(alg));
}


typedef void (*proc_t) (struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes);

static int process(int alg_id, int mode_id, proc_t proc, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	int rc;
	struct bc_alg alg;

	rc = bc_crypto_get_alg(alg_id, mode_id, &alg);
	if(0 != rc) {
		return rc;
	}

	rc = bc_crypto_set_key(&alg, key_handle);
	if(0 != rc) {
        bc_crypto_put_alg(&alg);
		return rc;
	}
	
	proc(&alg, iv, pool, data, nbytes);
	bc_crypto_put_alg(&alg);

	return 0;
}


int bc_encrypt(int alg_id, int mode_id, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	return process(alg_id, mode_id, bc_crypto_encrypt, key_handle, iv, pool, data, nbytes);
}


int bc_decrypt(int alg_id, int mode_id, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	return process(alg_id, mode_id, bc_crypto_decrypt, key_handle, iv, pool, data, nbytes);
}

==> BestCrypt-2.0.14.tar.gz: BestCrypt-2.0.14/kernel/kmod/bc_crypto.c <==
/*
 * Copyright (c) 2010 Jetico, Inc., Finland
 * All rights reserved.
 *
 * Interface to crypto library, cipher plugin reference counting, key storage.
 */

#include "BCKernel.h" // for mode definitions

#include "bc_crypto.h"
#include "bc_defs.h"
#include "bc_mod.h" 

#include "crypto/cipher.h"
#include "crypto/mode.h"

#include <linux/spinlock.h>
#include <linux/slab.h>

#include "bc_mem.h"

/**
 * Key table slot.
 * Designed for quick access when user locks the slot to copy key data somewhere, update pointers, etc.
 */
struct bc_key_slot {
    char*       key;
    size_t      keysize;
	spinlock_t	lock;	// lock to protect key addition and access ops.
};

#define BC_LOCK_INIT(slot) spin_lock_init(&(slot)->lock)
#define BC_LOCK_SLOT(slot) spin_lock(&(slot)->lock)
#define BC_UNLOCK_SLOT(slot) spin_unlock(&(slot)->lock)

// static key table data
static struct bc_key_slot g_key_table[kBCKernel_MaxKeys];


static inline int mode2tag(int mode_id) {
	switch (mode_id) {
	case kBCMode_CBC: return kCBC;
	case kBCMode_LRW: return kLRW;
	case kBCMode_XTS: return kXTS;
	default: return -1;
	};
}


static void* crypto_alloc(size_t nbytes) {
    void* mem = KMALLOC(nbytes);
    if(mem) {
        memset(mem, 0, nbytes);
    }

    return mem;
}


static void crypto_free(void* mem, size_t nbytes) {
    memset(mem, 0, nbytes);
    KFREE(mem);
}


static inline int is_free_slot(struct bc_key_slot* slot) {
	return (slot->key == NULL);
}


static inline void free_key_slot(struct bc_key_slot* slot) {
    crypto_free(slot->key, slot->keysize);
	slot->key = NULL;
	slot->keysize = 0;
}


/////////////////////////////////////////////////////////////////////////////////
//
//  Interface
//
/////////////////////////////////////////////////////////////////////////////////

static char g_zero_pool[512] = {0};

void bc_crypto_init_key_table(void)
{
	unsigned i;
	struct bc_key_slot* slot;

	memset(g_key_table, 0, sizeof(g_key_table));
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = g_key_table + i;
		BC_LOCK_INIT(slot);
	}	
}


void bc_crypto_release_key_table(void) 
{
	struct bc_key_slot* slot;
	unsigned i;
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = g_key_table + i;
		if(!is_free_slot(slot)) {
			BC_DEBUG_LOG("bestcrypt: warning: key slot %d not empty on free\n", i);
			free_key_slot(slot);
		}
	}
		
	memset(g_key_table, 0, sizeof(g_key_table));
}


int bc_crypto_store_key(char* key, size_t keysize, int* out_handle)
{
	struct bc_key_slot* slot = g_key_table;
	unsigned i;

	// we first lock and then check if the spot is free to make sure that two concurrent requests wont race for the same free slot.
	for(i = 0; i < kBCKernel_MaxKeys; ++i) {
		slot = &g_key_table[i];

		BC_LOCK_SLOT(slot);

		if(is_free_slot(slot)) {
			break;
		} else {
			BC_UNLOCK_SLOT(slot);
		}
	}

	if(i == kBCKernel_MaxKeys) {
		return -EBUSY;
	}

	slot->key = key;
	slot->keysize = keysize;

	BC_UNLOCK_SLOT(slot);

	*out_handle = i;
	
	return 0;
}

int bc_crypto_free_key(int handle)
{
	struct bc_key_slot* slot;

    if (handle >= kBCKernel_MaxKeys || handle < 0) {
        	return -EINVAL;
	}

	slot = &g_key_table[handle];

	BC_LOCK_SLOT(slot);

	if(is_free_slot(slot)) {
		BC_UNLOCK_SLOT(slot);
		return -EINVAL;
	}
	
	free_key_slot(slot);
	
    BC_UNLOCK_SLOT(slot);

	return 0;
}


int bc_crypto_get_alg(int alg_id, int mode_id, struct bc_alg* out_alg)
{
	int	mode_tag;
    const struct bc_mode*   mode;
    const struct bc_cipher* cipher;

    if (NULL == out_alg) {
        BC_KERN_ERROR("Invalid argument\n");
        return -EINVAL;
    }

	mode_tag = mode2tag(mode_id);
	if(mode_tag == -1) {
        BC_KERN_ERROR("No such mode id 0x%x\n", mode_id);
		return -EINVAL;
	}

	cipher = bc_get_plugin(alg_id);
	if(!cipher) {
        BC_KERN_ERROR("No such cipher id 0x%x\n", alg_id);
		return -EINVAL;
	}

	mode = bc_lookup_mode(mode_tag);
    if(!mode || !mode->is_supported(cipher)) {
        bc_put_plugin(cipher);
        BC_KERN_ERROR("Mode 0x%x is not supported by cipher 0x%x\n", mode_id, alg_id);
        return -EINVAL;
    }

	memset(out_alg, 0, sizeof(*out_alg));

    out_alg->key_size = mode->key_size(cipher);
    out_alg->extended_key_size = mode->context_size(cipher);
	out_alg->cipher = cipher;
	out_alg->mode = mode;
    out_alg->key = NULL;

	return 0;
}


int bc_crypto_set_key(struct bc_alg* alg, int handle)
{
    void* ctx;
	struct bc_key_slot* slot;

	if(handle < 0 || handle >= kBCKernel_MaxKeys || NULL == alg) {
		return -EINVAL;
	}

	slot = &g_key_table[handle];
	
	BC_LOCK_SLOT(slot);

	if(!is_free_slot(slot)) {
        
        if(slot->keysize < alg->key_size) {
            BC_UNLOCK_SLOT(slot);
            return -EINVAL;
        }

        ctx = crypto_alloc(alg->extended_key_size);
        if(!ctx) {
            BC_UNLOCK_SLOT(slot);
            return -EINVAL;
        }

        alg->mode->init_context(ctx, alg->cipher, slot->key, slot->keysize);

        if(alg->key) {
            crypto_free(alg->key, alg->extended_key_size);
        }

        alg->key = ctx;

	} else {
	    BC_UNLOCK_SLOT(slot);
		return -EINVAL;
	}

	BC_UNLOCK_SLOT(slot);

	return 0;
}


int bc_crypto_set_key_explicit(struct bc_alg* alg, char* key, size_t keysize)
{
    void* ctx;

    if(!alg || !key) {
        return -EINVAL;
    }

    if(keysize < alg->key_size) {
        return -EINVAL;
    }

    ctx = crypto_alloc(alg->extended_key_size);
    if(!ctx) {
        return -ENOMEM;
    }

    alg->mode->init_context(ctx, alg->cipher, key, keysize);

    if(alg->key) {
        crypto_free(alg->key, alg->extended_key_size);
    }

    alg->key = ctx;
    return 0;
}


void bc_crypto_encrypt(struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	size_t nblocks = nbytes / alg->cipher->block_size;
	alg->mode->encrypt(alg->key, alg->cipher, iv, (pool ? pool : g_zero_pool), data, nblocks);
}


void bc_crypto_decrypt(struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	size_t nblocks = nbytes / alg->cipher->block_size;
	alg->mode->decrypt(alg->key, alg->cipher, iv, (pool ? pool : g_zero_pool), data, nblocks);
}


void bc_crypto_put_alg(struct bc_alg* alg)
{
	bc_put_plugin(alg->cipher);
    if(alg->key) {
        crypto_free(alg->key, alg->extended_key_size);
    }

    memset(alg, 0, sizeof(*alg));
}


typedef void (*proc_t) (struct bc_alg* alg, uint64_t iv, char* pool, char* data, size_t nbytes);

static int process(int alg_id, int mode_id, proc_t proc, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	int rc;
	struct bc_alg alg;

	rc = bc_crypto_get_alg(alg_id, mode_id, &alg);
	if(0 != rc) {
		return rc;
	}

	rc = bc_crypto_set_key(&alg, key_handle);
	if(0 != rc) {
        bc_crypto_put_alg(&alg);
		return rc;
	}
	
	proc(&alg, iv, pool, data, nbytes);
	bc_crypto_put_alg(&alg);

	return 0;
}


int bc_encrypt(int alg_id, int mode_id, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	return process(alg_id, mode_id, bc_crypto_encrypt, key_handle, iv, pool, data, nbytes);
}


int bc_decrypt(int alg_id, int mode_id, int key_handle, uint64_t iv, char* pool, char* data, size_t nbytes)
{
	return process(alg_id, mode_id, bc_crypto_decrypt, key_handle, iv, pool, data, nbytes);
}
